/* 
Exemple 3: dessine un pixel rouge si le clic gauche est appuyé.
Quitte le programme si le clic droit est appuyé 

Fonction utile:

int SDL_GetMouseState(int* x, int* y);
cette fonction stocke dans x et y la position de la souris,
et renvoie un entier qui encode les boutons appuyés.
SDL contient trois valeurs particulières:
SDL_BUTTON_LMASK
SDL_BUTTON_RMASK
SDL_BUTTON_MMASK
correspondant aux boutons gauche, droit et milieu de la souris.
En pratique, ce sont des puissances de deux, ce qui permet
de savoir si un bouton est appuyé avec des opérations bit à bit.
Cela fonctionne selon le même principe que le DM1 sur les ensembles !
Par exemple:

int x, y;
int boutons = SDL_GetMouseState(&x, &y);
if (boutons & SDL_BUTTON_LMASK > 0){
	printf("Le clic gauche est appuyé\n");
}
if (boutons & SDL_BUTTON_RMASK > 0){
	printf("Le clic gauche est appuyé\n");
}
*/



#include <SDL.h>
#include <time.h>
#include <unistd.h>
#include <stdbool.h>

#define H 480
#define W 640

SDL_Window* window; // fenêtre du programme
SDL_Renderer* renderer; // moteur de rendu

int main(){
	// Initialiser SDL: la fonction SDL_Init renvoie 0 si tout s'est bien passé
	if (SDL_Init(SDL_INIT_VIDEO) != 0){
		printf("Erreur d'initialisation: %s\n", SDL_GetError());
		exit(1);
	}

	// Créer la fenêtre, au milieu de l'écran
	SDL_WindowFlags window_flags = SDL_WINDOW_MOUSE_FOCUS;
	window = SDL_CreateWindow(
		"Hello SDL", 
		SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED, 
		W, H, 
		window_flags
	);
	if (window == NULL){
		printf("Erreur de création de la fenêtre: %s\n", SDL_GetError());
		exit(1);
	}


	// Initialiser le moteur de rendu et le lier à la fenêtre
	renderer = SDL_CreateRenderer(window, -1, SDL_RENDERER_ACCELERATED);
	if (renderer == NULL){
		printf("Erreur de création du renderer: %s\n", SDL_GetError());
		exit(1);
	}



	/* INTERACTIONS UTILISATEUR 

    SDL fonctionne selon un système d'évènements. Chaque action de l'utilisateur
    (appuyer sur une touche, cliquer, etc...) crée un évènement.

    Il faut alors traiter ces évènements dans le code.

    L'exemple suivant affiche "Left" ou "Right" lorsque l'on appuie sur
    la touche correspondante du clavier, et dessine un pixel rouge à l'écran
    lorsque l'on clique sur l'écran.

	*/

	int x, y; // position de la souris

	bool pixels[H][W]; // savoir quels pixels ont été cliqués
	for (int i = 0; i < H; ++i){
		for (int j = 0; j < W; ++j){
			pixels[i][j] = false;
		}
	}

	bool quit = false; // booléen indiquant que l'utilisateur a cliqué sur la croix pour fermer la fenêtre
	while (!quit){

		SDL_Event event;

		// Les évènements sont stockés dans une file d'attente, et
		// la fonction SDL_WaitEvent pioche l'évènement en tête.
		// Si la file d'évènements est vide, elle va attendre de
		// recevoir un évènement
		SDL_WaitEvent(&event);
 
 		// la syntaxe switch case suivant permet de faire
 		// une sorte de match with simplifié en C, en exécutant
 		// différents blocs de code selon la valeur de event.type
        switch (event.type){
	        case SDL_QUIT: // évènement "utilisateur a quitté sur la croix"
	            quit = true;
	            break;

		    case SDL_KEYDOWN: // évènement "touche du clavier"
			    switch (event.key.keysym.sym)
			    {
			        case SDLK_LEFT:  printf("Left\n"); break;
			        case SDLK_RIGHT: printf("Right\n"); break;
			    }
  			    break;

  			case SDL_MOUSEBUTTONDOWN: // évènement "bouton de la souris"
			    switch (event.button.button)
			    {
			        case SDL_BUTTON_LEFT: // bouton gauche
			        	x = event.button.x;
			        	y = event.button.y;
			        	pixels[y][x] = true; // attention, x correspond aux colonnes et y aux lignes

						// réactualiser l'écran
						SDL_SetRenderDrawColor(renderer, 0, 0, 0, 255); 
						SDL_RenderClear(renderer);

			        	SDL_SetRenderDrawColor(renderer, 235, 100, 100, 255); // rouge
			        	for (int i = 0; i < H; ++i){
			        		for (int j = 0; j < W; ++j){
			        			if (pixels[i][j]){
			        				SDL_RenderDrawPoint(renderer, j, i);
			        			}
			        		}
			        	}
			        	SDL_RenderPresent(renderer);   
			    }
			    break;
	    }
	}

	SDL_DestroyWindow(window);
	SDL_Quit();

	return 0;
}